/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.client.internal.licensing");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.client.internal", "LicensingMessages");

var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.client.internal", "LicensingMessages");

// aliases and constants
var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var THIS_PACKAGE = com.ibm.team.repository.web.client.internal.licensing;
var LIC_ADMIN_SERVICE_URI = "com.ibm.team.repository.service.internal.ILicenseAdminRestService";
var LIC_CHECKER_SERVICE_URI = "com.ibm.team.repository.service.internal.ILicenseCheckerRestService";
var REPORT_SERVICE_URI = "com.ibm.team.reports.common.internal.service.IReportRestService";

//
// Usage examples
//
// Note, all function calls either an alias, e.g.
//     var getLicenseInfo = com.ibm.team.repository.web.client.internal.licensing.getLicenseInfo;
//     getLicenseInfo(...);
// or must be fully qualified...
//     com.ibm.team.repository.web.client.internal.licensing.getLicenseInfo(...);
//

// var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
//getLicenseInfo(new SRH(this, "successMethod", "onRemoteError"));
//getAssignedLicenses(new SRH(this, "successMethod", "onRemoteError"), {userId: "TestJazzGuest1"});
//assignLicense(new SRH(this, "successMethod", "onRemoteError"), {userId: "TestJazzGuest1", licenseId: "com.ibm.team.rtc.developer"});
//unassignLicense(new SRH(this, "successMethod", "onRemoteError"), {userId: "TestJazzGuest1", licenseId: "com.ibm.team.rtc.developer"});

//
// exported constants
//

// note: removed limitation of being 1 level away from context root
THIS_PACKAGE.FILE_UPLOAD_URI = dojo.baseUrl + "../../service/com.ibm.team.repository.service.internal.license.ILicenseUploadService";
THIS_PACKAGE.SERVER_COUNT_MAX_MSG = messages.serverCountMax;

//
// exported functions
//

// only meant to be used by HoverPopups!
function formatCALTypeDescription(CALType) {
	return "<div><div class=\"com-ibm-team-repository-CALPopup\"><p class=\"firstRow\"><span>" + messages.typeLabel + ":</span><span class=\"name\">" + CALType.name + "</span></p>" + CALType.description + "</div></div>";
}
THIS_PACKAGE.formatCALTypeDescription = formatCALTypeDescription;

// Returns true if server license is a trial.
function isServerLicenseTrial(licenseInfo) {
	return (licenseInfo.serverLicense && licenseInfo.serverLicense.expirationTime) ? true : false;
}
THIS_PACKAGE.isServerLicenseTrial = isServerLicenseTrial;

// Returns true if there are unassigned licenses of this CAL type and the CAL type is not disabled.
function isCALAvailable(CALType, isTrialServer){
	var count = computeCALAvailability(CALType, isTrialServer);
	return !(count == 0 || count == -2);
}
THIS_PACKAGE.isCALAvailable = isCALAvailable;

// Returns true if the type has non-disabled CALs available and the server will allow assignment to this
//  user.  To allow assignment to this user, either the user must already have CALs assigned or the server 
//  has not reached the maximum number of contributors.
function isCALAvailableForUser(CALType, licenseInfo, userHasNoCALs){
	if (userHasNoCALs && !isServerAcceptingNewLicensedUsers(licenseInfo) && CALType.countedInServerLimit){
		return false; // server can't accept new contributors
	} 
	return isCALAvailable(CALType, isServerLicenseTrial(licenseInfo));
}
THIS_PACKAGE.isCALAvailableForUser = isCALAvailableForUser;

// Returns true if the server has not reached the maximum number of contributors.
function isServerAcceptingNewLicensedUsers(licenseInfo){
	if (licenseInfo.serverLicense.maxContributors == -1) {
		return true;
	}
	return licenseInfo.serverLicense.maxContributors - licenseInfo.serverLicense.usedContributors >= 1;
}
THIS_PACKAGE.isServerAcceptingNewLicensedUsers = isServerAcceptingNewLicensedUsers;

// Returns the count of available licenses of this type.  
//  Result of -2 means this CAL type is disabled.
//  Result of -1 means the availability is unlimited.
function computeCALAvailability(CALType, isTrialServer) {
	if (CALType.disabledReason) {
		return -2; // CAL is disabled
	}
	if (CALType.total == -1) {
		return CALType.total; //CAL is unlimited
	}
	if (isTrialServer && (CALType.max == -1)) {
		return CALType.max; //CAL is unlimited
	}
	
	return isTrialServer ? CALType.max - CALType.used : CALType.total - CALType.used;
}
THIS_PACKAGE.computeCALAvailability = computeCALAvailability;

function sortCALs(types){
	return types.sort(function(a, b){
		// if one type is a functional type and the other isn't, the functional
		// one goes after
		if(a.functional !== b.functional) {
			return a.functional ? 1 : -1;
		}
		// otherwise sort on the name
		var aName = getCALNameInfo(a).toLowerCase();
		var bName = getCALNameInfo(b).toLowerCase();
		return aName > bName ? 1 : -1;
	});
}
THIS_PACKAGE.sortCALs = sortCALs;

// returns the string (""/Disabled/# available) for availability of a CAL type
function getCALAvailabilityString(CALType, isTrialServer, includeParens) {
	var available = computeCALAvailability(CALType, isTrialServer);
	var label = "";
	if (includeParens) {
		label = "(";
	}
	switch (available){
		case -1:
			label += messages.availableLabel;
			break;
		case -2:
			label += messages.deactivated;
			break;
		default : label += dojo.string.substitute(messages.availableCount, [available]);
	}
	if (includeParens) {
		label += ")";
	}
	return label;
}
THIS_PACKAGE.getCALAvailabilityString = getCALAvailabilityString;


function getCALNameInfo(CALType) {
	var nameString = "";
	if (CALType.productName) {
		nameString = CALType.productName;
	}
	if (CALType.productName && CALType.name) {
		nameString += " - ";
	}
	if (CALType.name) {
		nameString += CALType.name;
	}
	return nameString;
}
THIS_PACKAGE.getCALNameInfo = getCALNameInfo;

/**
 * Returns an object containing useful UI information for a Client 
 * Access License.
 */
function getClientAccessLicenseInfo(licenseInfo, CALType) {
	var isTrial = isServerLicenseTrial(licenseInfo);
	var availabilityLabel = getCALAvailabilityString(CALType, isTrial, true);
	var simpleAvailabilityString = getCALAvailabilityString(CALType, isTrial, false);
	var available = computeCALAvailability(CALType, isTrial);
	var name = getCALNameInfo(CALType);
	var assignableToUserWithoutCALs = isCALAvailableForUser(CALType, licenseInfo, true);
	var assignableToUserWithCALs = isCALAvailableForUser(CALType, licenseInfo, false);;
	
	return {
		availabilityLabel: availabilityLabel,
		simpleAvailabilityString: simpleAvailabilityString,
		available: available,
		name: name, 
		assignableToUserWithoutCALs: assignableToUserWithoutCALs, 
		assignableToUserWithCALs: assignableToUserWithCALs
	};
}
THIS_PACKAGE.getClientAccessLicenseInfo = getClientAccessLicenseInfo;

/**
 * Returns an object containing useful UI information related to a server
 * license's expiration time.
 * 
 * @param {String} expirationTime the RFC 3339 formatted time stamp contained in
 *  licenseInfo.serverLicense.expirationTime; do not send in null
 */
function getExpirationInfo(expirationTime) {
	
	// format expiration date string
	var expirationDate = dojo.date.stamp.fromISOString(expirationTime);
	var expirationDateString = dojo.date.locale.format(expirationDate, {formatLength: "long", selector: "date" });
	
	// determine if license is expired or not
	var now = new Date();
	var isExpired = now > expirationDate;
	
	// determine how many days remaining or since expiration
	var dayDiff = Math.abs(dojo.date.difference(now, expirationDate, "day"));
	var daysRemaining = isExpired ? null : dayDiff;
	var daysSinceExpiration = isExpired ? dayDiff : null;
	
	// return "Formatted Date (# days ago/remaining)" such as "March 1, 2009 (48 days remaining)"
	if (dayDiff === 0) {
		var relativeDetails = messages.today;
	} else if(isExpired) {
		var relativeDetails = dayDiff === 1 ? messages.dayAgo : dojo.string.substitute(messages.daysAgo, [dayDiff]);
	} else {
		var relativeDetails = dayDiff === 1 ? messages.dayRemaining : dojo.string.substitute(messages.daysRemaining, [dayDiff]);
	}	
	var expirationDateRelative = dojo.string.substitute(messages.dateAndDayCount, [expirationDateString, relativeDetails]);
	
	return {
		daysRemaining: daysRemaining,
		expirationDateString: expirationDateString,
		expirationDateRelative: expirationDateRelative,
		isExpired: isExpired
	};
}
THIS_PACKAGE.getExpirationInfo = getExpirationInfo;

//Returns true if any of the installed CALs have expiration dates
//Set: isFloating=true to check only the floating types for expired CALs
// isFloating=false for dedicated CAL types

function checkExpiringCALs(licenseInfo,isFloating){
	var self = this;
	var val = false;
	var types = (isFloating ? licenseInfo.floatingLicenseTypes : licenseInfo.types) || [];
	dojo.some(types, function(type){
		if (type.expirationTime != null) {
			val = true;
			return;
		}
	});
	return val;
}
THIS_PACKAGE.checkExpiringCALs = checkExpiringCALs;

function allowsPurchaseCountUpdates(CALType, isTrialServer, isFloating) {
	if (isTrialServer && !isFloating) {
		return false;
	}
	
	var max = CALType.max;
	if (CALType.max === -1) {
		max = Infinity;
	}
	var included = CALType.free;
	if (CALType.free === -1) {
		included = Infinity;
	}
	return max > included;
}
THIS_PACKAGE.allowsPurchaseCountUpdates = allowsPurchaseCountUpdates;


/**
 * See ILicenseAdminRestService#getLicenseInfo
 */
function getLicenseInfo(srh) {
	TeamServerClient.invokeService(getLicenseAdminServiceRequest("getLicenseInfo"), srh);
}
THIS_PACKAGE.getLicenseInfo = getLicenseInfo;

/**
 * See ILicenseAdminRestService#getDefaultLicenseId
 */
function getDefaultLicenseId(srh) {
	TeamServerClient.invokeService(getLicenseAdminServiceRequest("getDefaultLicenseId"), srh);
}
THIS_PACKAGE.getDefaultLicenseId = getDefaultLicenseId;

/**
 * See ILicenseAdminRestService#getUsedLicenseInfo
 */
function getUsedLicenseInfo(srh) {
	TeamServerClient.invokeService(getLicenseAdminServiceRequest("getUsedLicenseInfo"), srh);
}
THIS_PACKAGE.getUsedLicenseInfo = getUsedLicenseInfo;

/**
 * See ILicenseAdminRestService#getCheckLicense
 */
function checkLicense(srh, args) {
	TeamServerClient.invokeService(getLicenseCheckerServiceRequest("getCheckLicense", args), srh);
}
THIS_PACKAGE.checkLicense = checkLicense;

function checkFloatingLicensesSupported(yes_func, no_func, error_func) {
	var floatingLicenseSupported = {
		sucess: function(status) {
			if (status.code == "OK") {
				yes_func();
			} else {
				no_func(status);
			}		
		},
		failure: function(err) {
			error_func(err);
		}
	}
	checkLicense(new SRH(floatingLicenseSupported, "sucess", "failure"), {operationId:"com.ibm.team.repository.support.floatingLicenses"});
}
THIS_PACKAGE.checkFloatingLicensesSupported = checkFloatingLicensesSupported;

/**
 * See ILicenseAdminRestService#postTerminateLease
 */
function postTerminateLease(srh, args) {
	TeamServerClient.invokeService(getLicenseAdminServiceRequest("postTerminateLease", args), srh);
}
THIS_PACKAGE.postTerminateLease = postTerminateLease;

/**
 * See ILicenseAdminRestService#getAssignedLicenses
 */
function getAssignedLicenses(srh, args) {
	validateArgs("getAssignedLicenses", args, ["userId"]);
	TeamServerClient.invokeService(getLicenseAdminServiceRequest("getAssignedLicenses", args), srh);
}
THIS_PACKAGE.getAssignedLicenses = getAssignedLicenses;

var ASSIGN_ARGS = ["userId", "licenseId"];
var ASSIGN_MULTIPLE_ARGS = [ "licenseId", "jsonUserIds" ];
var REPORT_ARGS = [ "bundle", "path", "parameterName", "parameterValue", "showArchived"];

/**
 * See ILicenseAdminRestService#postAssignLicense
 */
function assignLicense(srh, args) {
	validateArgs("assignLicense", args, ASSIGN_ARGS);
	TeamServerClient.invokeService(getLicenseAdminServiceRequest("postAssignLicense", args), srh);
}
THIS_PACKAGE.assignLicense = assignLicense;

/**
 * See ILicenseAdminRestService#postAssignLicenseMultiple
 */
function assignLicenseMultiple(srh, args) {
	validateArgs("assignLicenseMultiple", args, ASSIGN_MULTIPLE_ARGS);
	TeamServerClient.invokeService(getLicenseAdminServiceRequest("postAssignLicenseMultiple", args), srh);
}
THIS_PACKAGE.assignLicenseMultiple = assignLicenseMultiple;

/**
 * See ILicenseAdminRestService#postUnassignLicense
 */
function unassignLicense(srh, args) {
	validateArgs("unAssignLicense", args, ASSIGN_ARGS);
	TeamServerClient.invokeService(getLicenseAdminServiceRequest("postUnassignLicense", args), srh);
}
THIS_PACKAGE.unassignLicense = unassignLicense;

function postLicenseInfo(srh, args) {
	args.jsonMaxCounts = dojo.toJson(args.maxCounts);
	delete args.maxCounts;
	if(args.serverFile === null) {
		delete args.serverFile;
	}
	TeamServerClient.invokeService(getLicenseAdminServiceRequest("postLicenseInfo", args), srh);
}
THIS_PACKAGE.postLicenseInfo = postLicenseInfo;

function getReportHtml(srh, args) {
	validateArgs("getReportHtml", args, REPORT_ARGS);
	TeamServerClient.invokeService(getReportServiceRequest("getRenderedReportFromBundle", args), srh);
}
THIS_PACKAGE.getReportHtml = getReportHtml;

//
// internal functions
//

function getLicenseAdminServiceRequest(method, args){
	return getServiceRequest(LIC_ADMIN_SERVICE_URI, method, args);
}

function getLicenseCheckerServiceRequest(method, args){
	return getServiceRequest(LIC_CHECKER_SERVICE_URI, method, args);
}

function getReportServiceRequest(method, args){
	return getServiceRequest(REPORT_SERVICE_URI, method, args);
}

function getServiceRequest(serviceUri, method, args) {
	if (args) {
		return new ServiceRequest(serviceUri, method, args);
	} else {
		return new ServiceRequest(serviceUri, method);
	}
}

function validateArgs(fName, args, props) {
	for(var i=0; i<props.length; i++) {
		if(!args[props[i]]) {
			throw new Error(fName + "missing required parameter args." + props[i]);
		}
	}
}

})();
